%% Basic Statistics in Matlab

%% 1. Import testScores
%Start with importing the data you want to run analysis on. This gives us
%the ages of all our participants, and their testScores, and their average response time (RT) on the test. 
%In this data set each person has been given the same test 7 times in total. Each row
%represents 1 participant. The 'RT' and 'age' matrices are arranged in the
%same way; 1 row per participant.

load 'testData.mat'

%% 2. Housekeeping 
%Start by pulling together some numbers you may want to refer to a few
%times in your sctipts. We can use the dimensions of the matrix
%'testScores' for this using the 'size' function.

noSubjects      =       size(testScores,1)
noTests         =       size(testScores,2)

%% 3. Organising the testScores for contrasts

%We can make some indexes based upon our hypotheses. We will use
%these for logical indexing later. Logical indexing was covered in Lecture
%3 'Importing and Handling testScores'.

%Let's say we are interested in looking at how older vs. young people score
%on our test. We want to start by splitting them into two groups, and
%making an index based on this.

splitMedian                    =     median(age)

groupIdx                       =     nan(noSubjects,1);  
groupIdx(age<splitMedian,1)    =     1;
groupIdx(age>=splitMedian,1)   =     2;

%When building indexes, a small mistake here can propragate through the
%rest of your script. Therefore it's good to check everything seems
%sensible. One good check would be to make sure we have an approximately equal amount of
%subjects in each group.

sum(groupIdx==1)
sum(groupIdx==2)

stats.groupN                       =    [sum(groupIdx==1) sum(groupIdx==2)]

%Later on, we make also want to plot data for each participant group, so we can define some ways
%to label it now

groupLabels                      = {'Young','Old'}
groupColours                    = {'red','blue'}
groups                          = [2];

%% 4. Descriptive Statistics
%If we want to compare how older vs younger people are doing on the tests,
%we can start by getting some descriptive statistics. Let's start by just
%looking at how they did on their baseline (first) test. Remember not to
%name you means 'mean' because you will overwrite the function 'mean'. If
%you accidentally overwrite it, just 'clear mean' and it will reset to
%matlab's default.

%we are just interested in the baseline test here, so we can make an index
%for which column this is in the test score matrix.

TEST        =       [1];

%We can get the mean of these test scores for the younger participants

mean        =       [mean(testScores(groupIdx==1,TEST))]
 
%now we can get the mean of the older participants, but we get an error!?!

mean        =       [mean(testScores(groupIdx==2,TEST))]
%this is because we labelled a variable 'mean' so we over wrote the
%function called 'mean' - oh no! But all is not lost, you can just clear
%the variable and it will reset to the default function as it is in the
%matlab path

clear mean
%and now this will work
mean(testScores(groupIdx==2,TEST))

%let's called the variable we want to save the mean in 'mu' instead. We can
%also get the standard deviation, and call them sigma.

mu                   =       [mean(testScores(groupIdx==1,TEST))...
                                    mean(testScores(groupIdx==2,TEST))];

sigma                =       [std(testScores(groupIdx==1,TEST))...
                                    std(testScores(groupIdx==2,TEST))];

%we can put them into a structure, as we know we will want to refer to
%these later, and include them when we write up our results
stats.mu(TEST,:)            =       mu;
stats.sigma(TEST,:)         =       sigma;

%% 5. Checking the distributions

%Before we run any statistical tests we want to know whether our test
%scores are normally distributed. Matlab has a bunch of different tests to look at
%the distributions, you can read the documentation about each of them and use
%your statistic knowledge to choose between them.

doc lillietest
doc adtest
doc jbtest

adtest(testScores(groupIdx==1,TEST))
adtest(testScores(groupIdx==2,TEST))

%You will see these return boolean logical answers. Many statistics
%tests in matlab will return this as a default. It tells you if you can
%reject the null hypothesis or not, where 'true' or [1] means you can
%reject the null hypothesis at a default alpha level (0.05). For most
%distribution tests the null hypothesis is that the data comes from a
%normal distribution, so if we are hoping our data will be from a normal
%distribution we do not want to reject the null.

%It is also nice to take a look at your data to check the distribution, so
%you could plot it as a histogram and eye-ball it to check there are no
%surprises.

figure;
subplot(1,2,1)
histogram(testScores(groupIdx==1,TEST),'FaceColor',groupColours{1});
title(groupLabels{1})

subplot(1,2,2)
histogram(testScores(groupIdx==2,TEST),'FaceColor',groupColours{2});
title(groupLabels{2})


%% 6. Two Sample Ttest
%Now we are happy our data is normally distributed, we know we can run
%parametric tests. Matlab has a function 'ttest2' which compares
%the distributions of two inputs and will tell you if they are significantly
%different (two sample ttest).

ttest2(testScores(groupIdx==1,TEST),testScores(groupIdx==2,TEST))

%Like adtest above just tells us whether the test is significantly different (the test decision). We also want to
%know the p value for this test. This is the second output from the
%function. 
[h p] = ttest2(testScores(groupIdx==1,TEST),testScores(groupIdx==2,TEST))

%We can also ask it just to return the p value, and not test decision. 

[~,p] = ttest2(testScores(groupIdx==1,TEST),testScores(groupIdx==2,TEST))

%But remember, by labelling the output 'p', we aren't telling the function we
%want it to return the p value, we are just labelling the 2nd output p. So
%if we put p as the name for the first output, we won't get what we
%expected.

[p] = ttest2(testScores(groupIdx==1,TEST),testScores(groupIdx==2,TEST))

%The function also gives a few more useful outputs, including the
%confidence intervals and degrees of freedom.

[h p ci tstats] = ttest2(testScores(groupIdx==1,TEST),testScores(groupIdx==2,TEST))

%However for the sake of this example we are just interested in the p value, so we would like to save this.
%We can put it into our stats structure.

[~,stats.pValue_betweenBaseline(TEST)] = ttest2(testScores(groupIdx==1,TEST),testScores(groupIdx==2,TEST))

%We can now use the mean and standard deviation to plot the contrast we did
%for our statistical test.

xData       =       [1:2];
yData       =       [mean(testScores(groupIdx==1,TEST))...
                            mean(testScores(groupIdx==2,TEST))]
                       
yData_se    =       [std(testScores(groupIdx==1,TEST))/sqrt(sum(groupIdx==1))...
                            std(testScores(groupIdx==2,TEST))/sqrt(sum(groupIdx==2))]
                        
%...or we can use the statistics we saved earlier                        
yData       =       [stats.mu(TEST,:)];

yData_se    =       [stats.sigma(TEST,:)./sqrt(stats.groupN)];

figure;
bar(xData,yData); hold on
errorbar(yData,yData_se,'.k')
set(gca,'XTickLabel',groupLabels)

%% 7. Paired Sample T-test

%We may also be interested in whether test scores improve between the first
%and final session, here we can use the 'ttest' function to run a paired
%sample ttest.

[~,p] = ttest(testScores(:,1),testScores(:,7))

%Be careful to use the right function for the right test, ttest or ttest2. If you accidentally
%use ttest2 for two distributions from the same population and they
%happen to be the same size, it won't give you an error. If you are unsure
%about this, check your statistics lectures.

%Some statistics matlab functions can be passed vectors, but this isn't
%straightforward and you need to be careful to not make mistakes. If you
%want to run a few ttests you can use a loop. As an example, let's compare
%the testscore at each of the time points it was given, with the baseline
%test score. 

for TEST = 1:noTests;
    
[~,pValues(TEST)] = ttest(testScores(:,1),testScores(:,TEST));   
        
end

stats.pValue_withinBaseline        =       pValues; clear pValues

%We could also run this for each of the groups seperately to see if both
%groups improve.

for TEST = 1:noTests;
    
[~,pValues_younger(TEST)]       =       ttest(testScores(groupIdx==1,1),testScores(groupIdx==1,TEST));   
[~,pValues_older(TEST)]         =       ttest(testScores(groupIdx==2,1),testScores(groupIdx==2,TEST));  
        
end

stats.pValue_withinBaseline_younger          =       pValues_younger;
stats.pValue_withinBaseline_older            =       pValues_older;


%% 9. General Linear Model

%We have seen above that the Young participants test scores seem in
%increase each time they take the test, but this doesn't seem to happen for
%older people. We could consider looking at their rate of change, by
%comparing the slopes for a general linear model. Matlab has a function
%'glmfit' for this. 

% Using a loop we can fit a general linear model to each participant.

for subj = 1:noSubjects;

    xData   =   1:noTests;
    yData   =   testScores(subj,:);
    
    [betas(:,subj)] = glmfit(xData,yData);
        
end

% For clarity, we can seperate the constant beta and slope beta into two new
%matrices.
betaSlope  =   betas(2,:);
betaConst  =   betas(1,:);

% Now we can plot the general linear model for each participant using
% different colours for older and younger participants.
figure;
for subj = 1:noSubjects; %... run for each subject
    
    if groupIdx(subj)==1 %...if they subject is in the younger group plot them in red
        
        plotColour = 'red';
        
    elseif groupIdx(subj)==2 %...if they subject is in the older group plot them in blue
        
        plotColour = 'blue';
        
    end
    
plot(xData*betaSlope(subj)+betaConst(subj),plotColour);hold on

end
xlim([0 8])
ylim([0 35])
axis square

% We can now run one sample t-tests on the slopes to see if either group
% has a slope significantly different from 0. A non-significant outcome here would mean they did not
% improve (or get worse) with repeated testing. 
[~,p] = ttest(betaSlope)
[~,p] = ttest(betaSlope(groupIdx==1))
[~,p] = ttest(betaSlope(groupIdx==2))

%We used the ttest function above to compare two measures from the same group (paired sample ttest).
% Here we will just pass it out input (the slopes) instead of two. If you
% only pass the ttest function one input, it will compare this distribution
% to the null hypothesis that the mean of the distribution is 0. You can
% prove this to yourself by adding a second input of a vector of zeros and
% running the test again.

input1      =       betaSlope(groupIdx==2);
input2      =       zeros(size(input1))

[~,p] = ttest(input1,input2)

%You can also use this if you wanted to test if the mean of the
%distribution is significantly different from another number, for example
%'1';

input1      =       betaSlope(groupIdx==2);
input2      =       ones(size(input1))

[~,p] = ttest(input1,input2)

%If we compared the distribution of input1 to it's own mean, then we the
%result would not be significant.

input1      =       betaSlope(groupIdx==2);
input2      =       ones(size(input1))*mean(input1);

[~,p] = ttest(input1,input2)

%% 9. Running Correlations

%We have run most of our statistics so far by splitting our test scores into two
%groups, but it may be that we can use age as a continious variable. We can
%use the function 'corr' for this which gives us pearson's R and the p
%value for this test.

[R pval] = corr(age,testScores(:,1))

figure;
scatter(age(groupIdx==1),testScores(groupIdx==1,1),'MarkerFaceColor',groupColours{1}); hold on
scatter(age(groupIdx==2),testScores(groupIdx==2,1),'MarkerFaceColor',groupColours{2});
xlabel('Age')
ylabel('Baseline Test Scores')

%% 10. Non-parametric Statistics

%Matlab also has lot's of non-parametric statistics. Let's start by looking
%at some more data we have which are there average response times. We can see that these are not
%normally distributed so we shouldn't use the ttest or pearson's R

adtest(RT)

figure;
histogram(RT)

%The function 'corr' has an option to run a spearman's test for
%non-parametric data. It has other inputs you can look at in the
%documentation.

[R pval] = corr(age,RT,'type','spearman')

%There is also a function for performing a Wilcoxon signed-rank test. But
%it has a fun quirk! The first two output's of this test are in the reverse
%order to the ttest or ttest2 function. So remember this when you are
%labelling your outputs.

[p h] = signrank(RT(groupIdx==1),RT(groupIdx==2))

[p h] = ttest(RT(groupIdx==1),RT(groupIdx==2))

